@@ -1,7 +1,7 @@
 /* 
  * ml.c
  *
- * (c) 2008-2013  Klaus Schliep (klaus.schliep@gmail.com)
+ * (c) 2008-2014  Klaus Schliep (klaus.schliep@gmail.com)
  * 
  * 
  * This code may be distributed under the GNU GPL
@@ -15,7 +15,9 @@
 #include <R.h> 
 #include <R_ext/Lapack.h>
 #include <Rinternals.h>
-
+#ifdef _OPENMP
+# include <omp.h>
+#endif
 
 
 #define LINDEX(i, k) (i - ntips - 1L) * (nr * nc) + k * ntips * (nr * nc)
@@ -32,9 +34,9 @@ const double ScaleMAX = 4294967296.0;
 
 // 2^64 = 18446744073709551616
 
-static double *LL, *ROOT, *XX, *weight;
-static int *SCM, *X;
-
+static double *LL, *ROOT;
+static int *SCM, *XXX;
+    
 void ll_free(){
     free(LL);
     free(SCM);
@@ -56,29 +58,91 @@ void ll_init(int *nr, int *nTips, int *nc, int *k)
 }
 
 
-
+// contrast und nr,nc,k
 void ll_free2(){
     free(LL);
     free(SCM);
-    free(weight);
-    free(X);
+    free(ROOT);
+    free(XXX);
 }
 
 
-void ll_init2(int *XX, double *W, int *nr, int *nTips, int *nc, int *k)
+void ll_init2(int *data, int *weights, int *nr, int *nTips, int *nc, int *k)
 {   
     int i;
     LL = (double *) calloc(*nr * *nc * *k * *nTips, sizeof(double));
-    weight = (double *) calloc(*nr, sizeof(double));
+    ROOT = (double *) calloc(*nr * *nc * *k, sizeof(double));
+    XXX = (int *) calloc(*nr * *nTips, sizeof(int));
     SCM = (int *) calloc(*nr * *k * *nTips, sizeof(int));  // * 2L
-    X = (int *) calloc(*nr * *nTips, sizeof(int));
     for(i =0; i < (*nr * *k * *nTips); i++) SCM[i] = 0L;
-    for(i =0; i < *nr; i++) weight[i] = W[i];
-    for(i =0; i < (*nr * *nTips); i++) XX[i] = X[i];    
+    for(i =0; i < (*nr * *nTips); i++) XXX[i] = data[i];
 }
 
 
+/*
+void ll_init2(int *data, int *weights, int *nr, int *nTips, int *nc, int *k)
+{   
+    int i;
+    LL = (double *) calloc(*nr * *nc * *k * *nTips, sizeof(double));
+    ROOT = (double *) calloc(*nr * *nc * *k, sizeof(double));
+    XXX = (int *) calloc(*nr * *nTips, sizeof(int));
+    SCM = (int *) calloc(*nr * *k * *nTips, sizeof(int));  // * 2L
+    for(i =0; i < (*nr * *k * *nTips); i++) SCM[i] = 0L;
+    for(i =0; i < (*nr * *nTips); i++) XXX[i] = data[i];
+}
+*/
 
+void ll_init3(int *data, int *weights, int *nr, int *nTips, int *nc, int *k)
+{   
+    int i;
+    LL = (double *) calloc(*nr * *nc * *k * *nTips, sizeof(double));
+    ROOT = (double *) calloc(*nr * *nc * *k, sizeof(double));
+    XXX = (int *) calloc(*nr * *nTips, sizeof(int));
+    SCM = (int *) calloc(*nr * *k * *nTips, sizeof(int));  // * 2L
+    for(i =0; i < (*nr * *k * *nTips); i++) SCM[i] = 0L;
+    for(i =0; i < (*nr * *nTips); i++) XXX[i] = data[i];
+}
+
+
+struct Cdata{
+    int nr;
+    int nc;
+    int nrco;
+    int *data;
+    int *contrast;
+    double *weight;
+    double *LL;
+    double *LLL;
+    int *SCM; 
+};
+
+/*
+typedef struct Cdata partition;
+
+void init_Cdata(int *data, int *weights, int *nr, int *nTips, int *nc, int *k)
+{   
+    int i;
+    LL = (double *) calloc(*nr * *nc * *k * *nTips, sizeof(double));
+    ROOT = (double *) calloc(*nr * *nc * *k, sizeof(double));
+    XXX = (int *) calloc(*nr * *nTips, sizeof(int));
+    SCM = (int *) calloc(*nr * *k * *nTips, sizeof(int));  // * 2L
+    for(i =0; i < (*nr * *k * *nTips); i++) SCM[i] = 0L;
+    for(i =0; i < (*nr * *nTips); i++) XXX[i] = data[i];
+}
+*/
+
+SEXP getXX(SEXP nr, SEXP nTips){
+    int i, rws=INTEGER(nr)[0], nt=INTEGER(nTips)[0];  
+//    SEXP DAT;
+    int res;
+//    PROTECT(DAT = allocMatrix(INTSXP, rws, nt));
+//    for(i=0; i< rws*nt; i++) INTEGER(DAT)[i] = XXX[i];
+     res=0L;
+     for(i=0; i< rws*nt; i++) res += XXX[i];
+//    UNPROTECT(1);
+//    return(DAT);
+    return(ScalarInteger(res));
+}
 
 void matm(int *x, double *contrast, int *nr, int *nc, int *nco, double *result){
     int i, j;
@@ -161,7 +225,7 @@ SEXP getPM(SEXP eig, SEXP nc, SEXP el, SEXP w){
     for(j=0; j<nel; j++){ 
         for(i=0; i<nw; i++){
             PROTECT(P = allocMatrix(REALSXP, m, m));
-            if(edgelen[j]==0.0){
+            if(edgelen[j]==0.0 || ws[i]==0.0){
                 for(k=0; k<(m*m);k++)REAL(P)[k]=0.0;
                 for(k=0; k<m; k++)REAL(P)[k+k*m]=1.0;
             }
@@ -211,6 +275,9 @@ void lll(SEXP dlist, double *eva, double *eve, double *evei, double *el, double
 }
 
  
+// neue Version 
+// keine SEXP (dlist) 
+//  Ziel: openMP fuer Gamma (4 mal schneller)
 void lll0(int *X, double *eva, double *eve, double *evei, double *el, double g, int *nr, int *nc, int *node, int *edge, int nTips, double *contrast, int nco, int n, int *scaleTmp, double *bf, double *TMP, double *ans){
     int  ni, ei, j, i, rc; //    R_len_t i, n = length(node);
     double *rtmp, *P;
@@ -221,14 +288,13 @@ void lll0(int *X, double *eva, double *eve, double *evei, double *el, double g,
     P = (double *) R_alloc(*nc * *nc, sizeof(double));
 
     for(j=0; j < *nr; j++) scaleTmp[j] = 0L;
-// openMP statement
     for(i = 0; i < n; i++) {
         getP(eva, eve, evei, *nc, el[i], g, P); 
         ei = edge[i]; 
         if(ni != node[i]){
             if(ni>0)scaleMatrix(&ans[ni * rc], nr, nc, scaleTmp); // (ni-nTips)
             ni = node[i];
-            if(ei < nTips) 
+            if(ei < nTips)             
                 matp(&X[ei * *nr], contrast, P, nr, nc, &nco, &ans[ni * rc]); 
             else 
                 F77_CALL(dgemm)(transa, transb, nr, nc, nc, &one, &ans[(ei-nTips) * rc], nr, P, nc, &zero, &ans[ni * rc], nr);
@@ -289,8 +355,91 @@ void lll3(SEXP dlist, double *eva, double *eve, double *evei, double *el, double
     F77_CALL(dgemv)(transa, nr, nc, &one, &ans[ni * rc], nr, bf, &ONE, &zero, TMP, &ONE);
 }
 
+// ohne openMP
+SEXP PML_NEW2(SEXP EL, SEXP W, SEXP G, SEXP NR, SEXP NC, SEXP K, SEXP eig, SEXP bf, SEXP node, SEXP edge, SEXP NTips, SEXP root, SEXP nco, SEXP contrast, SEXP N){
+    int nr=INTEGER(NR)[0], nc=INTEGER(NC)[0], k=INTEGER(K)[0], i, indLL; 
+    int nTips = INTEGER(NTips)[0], *SC;
+//    int *nodes=INTEGER(node), 
+    double *g=REAL(G), *tmp, logScaleEPS;
+    SEXP TMP;
+    
+    double *eva, *eve, *evei;
+ 
+    eva = REAL(VECTOR_ELT(eig, 0));
+    eve = REAL(VECTOR_ELT(eig, 1));
+    evei = REAL(VECTOR_ELT(eig, 2));
+    
+    SC = (int *) R_alloc(nr * k, sizeof(int));   
 
 
+
+    PROTECT(TMP = allocMatrix(REALSXP, nr, k)); // changed
+    tmp=REAL(TMP);
+    for(i=0; i<(k*nr); i++)tmp[i]=0.0;
+    indLL = nr * nc * nTips;  
+    for(i=0; i<k; i++){                  
+        lll0(XXX, eva, eve, evei, REAL(EL), g[i], &nr, &nc, INTEGER(node), INTEGER(edge), nTips, REAL(contrast), INTEGER(nco)[0], INTEGER(N)[0], &SC[nr * i], REAL(bf), &tmp[i*nr], &LL[indLL *i]);           
+     } 
+
+    logScaleEPS = log(ScaleEPS);
+    for(i=0; i<(k*nr); i++) tmp[i] = logScaleEPS * SC[i] + log(tmp[i]);     //log
+    
+    UNPROTECT(1);
+    return TMP;     
+}
+
+// mit openMP
+SEXP PML_NEW(SEXP EL, SEXP W, SEXP G, SEXP NR, SEXP NC, SEXP K, SEXP eig, SEXP bf, SEXP node, SEXP edge, SEXP NTips, SEXP root, SEXP nco, SEXP contrast, SEXP N){
+    int nr=INTEGER(NR)[0], nc=INTEGER(NC)[0], k=INTEGER(K)[0], i, indLL, n=INTEGER(N)[0], ncontr=INTEGER(nco)[0]; 
+    int nTips = INTEGER(NTips)[0], *SC;
+    int *nodes=INTEGER(node), *edges=INTEGER(edge);
+    double *el=REAL(EL), *bfreq=REAL(bf), *contr=REAL(contrast);    
+    double *g=REAL(G), *tmp, logScaleEPS;
+    SEXP TMP;
+    
+    double *eva, *eve, *evei;
+ 
+    eva = REAL(VECTOR_ELT(eig, 0));
+    eve = REAL(VECTOR_ELT(eig, 1));
+    evei = REAL(VECTOR_ELT(eig, 2));
+    
+    SC = (int *) R_alloc(nr * k, sizeof(int));   
+
+    PROTECT(TMP = allocMatrix(REALSXP, nr, k)); // changed
+    tmp=REAL(TMP);
+    for(i=0; i<(k*nr); i++)tmp[i]=0.0;
+    indLL = nr * nc * nTips;
+    #pragma omp parallel for private(i)
+    for(i=0; i<k; i++){                  
+        lll0(XXX, eva, eve, evei, el, g[i], &nr, &nc, nodes, edges, nTips, contr, ncontr, n, &SC[nr * i], bfreq, &tmp[i*nr], &LL[indLL *i]);           
+     } 
+
+    logScaleEPS = log(ScaleEPS);
+    for(i=0; i<(k*nr); i++) tmp[i] = logScaleEPS * SC[i] + log(tmp[i]);     //log
+    
+    UNPROTECT(1);
+    return TMP;     
+}
+/*
+ * aus pml.fit 
+     ind = which(ll.0>0) 
+    sca = .Call(rowMax, resll, length(weight), as.integer(k)) + 1  
+##     sca auf SC
+    
+    lll = resll - sca  
+    lll <- exp(lll)  ###  exp raus
+    lll <- (lll%*%w)
+    lll[ind] = lll[ind] + exp(log(ll.0[ind])-sca[ind])    ### log
+    siteLik <- lll 
+    siteLik <- log(siteLik) + sca                         ### log
+    # needs to change
+    if(wMix >0) siteLik <- log(exp(siteLik) * (1-wMix) + llMix )
+    loglik <- sum(weight * siteLik)
+    if(!site) return(loglik)
+    resll = exp(resll)                                    ### exp    
+    return(list(loglik=loglik, siteLik=siteLik, resll=resll))  
+ */
+
 SEXP PML3(SEXP dlist, SEXP EL, SEXP W, SEXP G, SEXP NR, SEXP NC, SEXP K, SEXP eig, SEXP bf, SEXP node, SEXP edge, SEXP NTips, SEXP root, SEXP nco, SEXP contrast, SEXP N){
     int nr=INTEGER(NR)[0], nc=INTEGER(NC)[0], k=INTEGER(K)[0], i, indLL; 
     int nTips = INTEGER(NTips)[0], *SC;
@@ -318,50 +467,19 @@ SEXP PML3(SEXP dlist, SEXP EL, SEXP W, SEXP G, SEXP NR, SEXP NC, SEXP K, SEXP ei
      return TMP;     
 }
 
-/*
-SEXP PML02(SEXP dlist, SEXP EL, SEXP W, SEXP G, SEXP NR, SEXP NC, SEXP K, SEXP eig, SEXP bf, SEXP node, SEXP edge, SEXP NTips, SEXP root, SEXP nco, SEXP contrast, SEXP N){
-    int nr=INTEGER(NR)[0], nc=INTEGER(NC)[0], k=INTEGER(K)[0], i, indLL; 
-    int nTips = INTEGER(NTips)[0], *SC;
-    double *g=REAL(G), *tmp, logScaleEPS;
-    SEXP TMP;
-    
-    double *eva, *eve, *evei;
- 
-    eva = REAL(VECTOR_ELT(eig, 0));
-    eve = REAL(VECTOR_ELT(eig, 1));
-    evei = REAL(VECTOR_ELT(eig, 2));
-    
-    SC = (int *) R_alloc(nr * nc, sizeof(int)); 
-
-
-    PROTECT(TMP = allocMatrix(REALSXP, nr, k)); // changed
-    tmp=REAL(TMP);
-    for(i=0; i<(k*nr); i++)tmp[i]=0.0;
-    indLL = nr * nc * nTips;  
-    for(i=0; i<k; i++){                  
-        lll(dlist, eva, eve, evei, REAL(EL), g[i], &nr, &nc, INTEGER(node), INTEGER(edge), nTips, REAL(contrast), INTEGER(nco)[0], INTEGER(N)[0], &SC[nr * i], REAL(bf), &tmp[i*nr], &LL[indLL *i]);           
-     } 
-
-    logScaleEPS = log(ScaleEPS);
-
-     UNPROTECT(1);
-     return TMP;     
-}
-*/
 
 SEXP PML0(SEXP dlist, SEXP EL, SEXP W, SEXP G, SEXP NR, SEXP NC, SEXP K, SEXP eig, SEXP bf, SEXP node, SEXP edge, SEXP NTips, SEXP root, SEXP nco, SEXP contrast, SEXP N){
     int nr=INTEGER(NR)[0], nc=INTEGER(NC)[0], k=INTEGER(K)[0], i, indLL; 
     int nTips = INTEGER(NTips)[0], *SC;
     double *g=REAL(G), *tmp, logScaleEPS;
     SEXP TMP;
-    
     double *eva, *eve, *evei;
  
     eva = REAL(VECTOR_ELT(eig, 0));
     eve = REAL(VECTOR_ELT(eig, 1));
     evei = REAL(VECTOR_ELT(eig, 2));
     
-    SC = (int *) R_alloc(nr * nc, sizeof(int));   
+    SC = (int *) R_alloc(nr * k, sizeof(int));   
 
     PROTECT(TMP = allocMatrix(REALSXP, nr, k)); // changed
     tmp=REAL(TMP);
@@ -378,55 +496,6 @@ SEXP PML0(SEXP dlist, SEXP EL, SEXP W, SEXP G, SEXP NR, SEXP NC, SEXP K, SEXP ei
      return TMP;     
 }
 
-/*
-void pml0(int *X, double *eva, double *eve, double *evei, double el, double *g, int nr, int nc, int *node, int *edge, int nTips, double *contrast, int nco, 
-    int N, double *bf, double *LL){
-    int *SC, *sc, h, x;
-    double *tmp;
-    SC = (int *) R_alloc(nr * nc, sizeof(int));  
-    sc = (int *) R_alloc(nr, sizeof(int));
-    tmp = (double *) R_alloc(*nr * *nc, sizeof(double)); 
-    PROTECT(TMP = allocMatrix(REALSXP, nr, k)); // changed
-    tmp=REAL(TMP);
-    for(i=0; i<(k*nr); i++)tmp[i]=0.0;
-    indLL = nr * nc * nTips;  
-    for(i=0; i<k; i++){                  
-        lll0(X, eva, eve, evei, el, g[i], &nr, &nc, INTEGER(node), INTEGER(edge), nTips, REAL(contrast), INTEGER(nco)[0], INTEGER(N)[0], &SC[nr * i], REAL(bf), &tmp[i*nr], &LL[indLL *i]);           
-     } 
-    logScaleEPS = log(ScaleEPS);
-    for(i=0; i<(k*nr); i++) tmp[i] = logScaleEPS * SC[i] + log(tmp[i]);     
-      
-    for(i = 0; i < *nr; i++){  
-        x = SC[i];
-        for(h = 1; h< k; h++) {if(SC[i + h* *nr] < x) x=dat[i + h* *nr];}
-        for(h = 1; h< k; h++){ SC[i + h* *nr] -= x;}
-        sc[i] = x;               
-        }
-    for(i=0; i<(k*nr); i++) tmp[i] = logScaleEPS * SC[i] + log(tmp[i]);         
-// SC = SC -sc 
-
-
-    ind = which(ll.0>0)
-# ind uebergeben    
-#    sca = .Call(rowMax, resll, length(weight), as.integer(k)) + 1  
-#    lll = resll - sca  
-    lll <- exp(lll) 
-    lll <- (lll%*%w)
-    
-    lll[ind] = lll[ind] + exp(log(ll.0[ind])-sca[ind])    
-    
-    siteLik <- lll 
-    siteLik <- log(siteLik) + sca
-    # needs to change    
-    loglik <- sum(weight * siteLik)
-    if(wMix >0) siteLik <- log(exp(siteLik) * (1-wMix) + llMix )
-    
-    if(!site) return(loglik)
-    resll = exp(resll) 
-    return(list(loglik=loglik, siteLik=siteLik, resll=resll)) 
-
-}
-*/
 
 
 // replace child with LL
@@ -675,7 +744,7 @@ SEXP getROOT(SEXP ax, SEXP nrx, SEXP ncx){
     return(RES);
 }
 
-
+/*
 SEXP getXX(SEXP ax, SEXP nrx, SEXP ncx){
     int j, nc = INTEGER(ncx)[0], nr = INTEGER(nrx)[0], a = INTEGER(ax)[0];
     SEXP RES;
@@ -684,6 +753,7 @@ SEXP getXX(SEXP ax, SEXP nrx, SEXP ncx){
     UNPROTECT(1);
     return(RES);
 }
+*/
 
 // ch * (pa %*% P)  
 void getMI(int child, int parent, double el, double *eva, double *eve, double *evi, double *g, int nr, int nc, int k, int ntips){
@@ -800,7 +870,7 @@ SEXP LogLik2(SEXP dlist, SEXP P, SEXP nr, SEXP nc, SEXP node, SEXP edge, SEXP nT
     return(ans);
 }
 
-
+//raus
 static R_INLINE void matprod(double *x, int nrx, int ncx, double *y, int nry, int ncy, double *z)
 {
     F77_CALL(dgemm)(transa, transb, &nrx, &ncy, &ncx, &one, x, &nrx, y, &nry, &zero, z, &nrx);
@@ -846,7 +916,7 @@ SEXP FS4(SEXP eig, SEXP nc, SEXP el, SEXP w, SEXP g, SEXP X, SEXP dad, SEXP chil
     while ( (eps > 1e-05) &&  (k < 5) ) {
         if(scalep>0.6){  
             NR55(eva, ncx-1L, edle, ws, gs, X, INTEGER(ld)[0], nrx, f, tmp);  
-            ll=0.0;  
+            ll=0.0; 
             lll=0.0;        
 //            for(i=0; i<nrx ;i++) ll+=wgt[i]*tmp[i];
 //            for(i=0; i<nrx ;i++) lll+=wgt[i]*tmp[i]*tmp[i];  
@@ -896,7 +966,7 @@ SEXP FS4(SEXP eig, SEXP nc, SEXP el, SEXP w, SEXP g, SEXP X, SEXP dad, SEXP chil
     return (RESULT);
 } 
 
-// eva statt eig uebergeben
+
 SEXP FS5(SEXP eig, SEXP nc, SEXP el, SEXP w, SEXP g, SEXP X, SEXP ld, SEXP nr, SEXP basefreq, SEXP weight, SEXP f0)
 {
     SEXP RESULT; // EL, P; 
