@@ -52,6 +52,36 @@ void get_two_index_integer(int *x, int *val, int *index)
 */
 
 
+void countCycle(int *M, int *l, int *m, int *res){
+    int j, i, tmp;
+    res[0]=0L;
+    for (i=0; i<*l; i++) {
+        tmp = 0;
+        if(M[i] != M[i + (*m -1) * *l])tmp++;
+        for (j=1; j<*m; j++) {
+            if(M[i + (j-1)* *l] != M[i + j * *l])tmp++;            
+        }
+        if(tmp>2L)res[0]+=tmp;
+    }
+}
+
+
+void countCycle2(int *M, int *l, int *m, int *res){
+    int j, i, tmp;
+//    res[0]=0L;
+    for (i=0; i<*l; i++) {
+//        res[i] = 0L;
+        tmp = 0L;
+        if(M[i] != M[i + (*m -1) * *l])tmp=1L;
+        for (j=1; j<*m; j++) {
+            if(M[i + (j-1L)* *l] != M[i + j * *l])tmp++;            
+        }
+        res[i]=tmp;
+    }
+}
+
+
+
 void nodeH(int *edge, int *node, double *el, int *l,  double *res){
     int ei, i;
     for (i=*l-1L; i>=0; i--) {
@@ -166,8 +196,6 @@ void getd2P2(double *eva, double *ev, double *evi, int m, double el, double w, d
 }
 
 
-
-
 SEXP getPM2(SEXP eig, SEXP nc, SEXP el, SEXP w){
     R_len_t i, j, nel, nw;
     int m=INTEGER(nc)[0], l=0;
@@ -315,11 +343,12 @@ SEXP getd2PM2(SEXP eig, SEXP nc, SEXP el, SEXP w){
     return(RESULT);
 } 
 
-    
+
+/*
 static R_INLINE void emult(double *x, double *y, int n){
     for(int i=0; i<n; i++) x[i]*=y[i];
 }
-
+*/
 
 
 void tabulate(int *x, int *n, int *nbin, int *ans){
@@ -383,8 +412,11 @@ SEXP AllChildren(SEXP children, SEXP parent, SEXP M){
     SEXP RESULT, TMP;
     tab = (int*)R_alloc(m, sizeof(int));
     for(i=0; i<m; i++)tab[i]=0L;
-    
-    j=0;
+//  tmp = tab
+
+
+
+    j=0;    
     p = INTEGER(parent)[0];
     for(i=0; i<n; i++){
         if(INTEGER(parent)[i]!=p){
@@ -393,6 +425,8 @@ SEXP AllChildren(SEXP children, SEXP parent, SEXP M){
         } 
         tab[j] += 1L;
     }
+
+//    for(i=0; i<n; i++) tab[INTEGER(parent)[i] - 1L] ++;  // 7 Zeilen weniger      
     PROTECT(RESULT = allocVector(VECSXP, m));
 
     i=0L;    
@@ -412,6 +446,74 @@ SEXP AllChildren(SEXP children, SEXP parent, SEXP M){
     return(RESULT);
 }
 
+/*
+ *
+ 
+setwd("/home/klaus/Desktop/phangorn/src/") 
+allKids <- function(phy){
+    nTips = as.integer(length(phy$tip))
+    lp=nrow(phy$edge)
+    nNode = phy$Nnode
+    .C("AllKids", as.integer(phy$edge[,2]), as.integer(phy$edge[,1]), as.integer(nTips), 
+        as.integer(nNode), as.integer(lp), integer(lp), integer(nNode+1L),integer(nNode))
+}
+
+
+   
+bipHelp <- function(phy){
+    nTips = as.integer(length(phy$tip))
+    lp=nrow(phy$edge)
+    nNode = phy$Nnode
+    mp = max(phy$edge)
+    .C("bipHelp", as.integer(phy$edge[,1]), as.integer(phy$edge[,2]), as.integer(nTips), 
+        as.integer(mp), as.integer(lp), integer(mp), integer(mp+1L))
+} 
+ 
+ 
+setwd("/home/klaus/Desktop/phangorn/src/") 
+dyn.load("phangorn.so")
+library(phangorn)
+
+tree = rtree(8)
+tree = reorder(tree, "postorder")
+bipHelp(tree)
+
+
+allKids(tree)
+
+
+
+*/
+
+void AllKids(int *children, int *parents, int *nTips, int *nNode, int *lp, int *kids, int *lkids, int *pkids){
+    int i, k, m=nNode[0], p; // l=0L, *tab , j 
+    int n=lp[0]; 
+    for(i=0; i<m; i++){
+        pkids[i]=0L;
+        lkids[i]=0L;
+    }
+    for(i=0; i<lp[0]; i++)kids[i]=0L;
+//    j=0;
+    p = 0L;
+    for(i=0; i<n; i++){
+        p = parents[i] - 1L - nTips[0];
+        pkids[p] += 1L;
+    }
+    for(i=0; i<*nNode; i++)lkids[i+1] = lkids[i] + pkids[i];
+    
+    i=0L;   
+    k=0;
+    p=0L;
+    for(i=0; i<n; i++){
+        if(parents[i]!=p){
+            p=parents[i];
+            k=lkids[p- nTips[0] -1L];
+        }
+        else k++;
+        kids[k] = children[i];
+    }
+    
+}
 
 
 /*
@@ -591,6 +693,196 @@ void cisort2(int *x, int *y, int a, int b, int *res){
     }
 }    
 
+// faster cophenetic 
+
+void C_bipHelp(int *parents, int *children, int *ntips, int *mp, int *l, int *ltips, int *ptips){
+   int p, k, i;
+   for(i=0; i<*ntips; i++)ltips[i]=1L;
+   for(i=*ntips; i<*mp; i++)ltips[i]=0L;
+   for(i=0; i<*l; i++){
+       p = parents[i]-1L;
+       k = children[i]-1L;
+       ltips[p]+=ltips[k];
+   }
+   for(i=0; i<(*mp+1); i++)ptips[i]=0L;
+   for(i=0; i<*mp; i++)ptips[i+1]=ptips[i] + ltips[i];
+}   
+
+
+void C_bip2(int *parents, int *children, int *ntips, int *mp, int *l, int *ltips, int *ptips, int *tips){
+    int eins=1L, i, j, p, pi, ci, ltmp; 
+    int *tmp, *tmp2;  
+    tmp = (int *) R_alloc(*mp, sizeof(int));
+    tmp2 = (int *) R_alloc(*mp, sizeof(int));
+    for(i=0; i<*ntips; i++)tips[i]=i+1L;
+    for(i=*ntips; i<ptips[*mp]; i++)tips[i]=0L; 
+    p=parents[0];
+
+    tmp[0] = 0L; //children[0]; 
+    ltmp=  0L; //1L;
+    for(i=0; i<*l; i++){ 
+        pi = parents[i]; 
+        ci = children[i];
+        if(pi==p){
+             if(ci < (*ntips+1L)){
+                 cisort(&ci, tmp, &eins, &ltmp, tmp2);            
+                 ltmp += 1L;
+                 for(j=0; j<ltmp; j++) tmp[j] = tmp2[j];
+             }
+             else{
+                 cisort(&tips[ptips[ci-1L]], tmp, &(ltips[ci-1L]), &ltmp, tmp2);                       
+                 ltmp += ltips[ci-1L]; //  lch[ci]; 
+//               ltmp +=   lch[ci];
+                 for(j=0; j<ltmp; j++) tmp[j] = tmp2[j];                                
+             } 
+//             kl[pi]=k; 
+//             lch[pi] = ltmp;
+        }  
+        else{
+            for(j=0; j<ltmp; j++) tips[ptips[p-1L]+j] = tmp2[j];//tmp2[j]
+            if(ci < (*ntips+1)){ 
+                 tmp[0]=ci;
+                 ltmp=1L; 
+            } 
+            else{ 
+                ltmp=ltips[ci-1L];
+                for(j=0; j<ltmp; j++)tmp[j] = tips[ptips[ci-1L]+j]; // , ci-1L))[j];
+            }
+//            k += 1L; 
+            p = pi;
+        }
+    }
+    for(j=0; j<ltmp; j++) tips[ptips[p-1L]+j] = tmp2[j];
+}   
+
+// doppelt
+int give_index3(int i, int j, int n)
+{
+    if (i > j) return(DINDEX(j, i));
+    else return(DINDEX(i, j));
+}
+
+// faster and less memory consuming cophenetic
+void copheneticHelp(int *left, int *right, int *ll, int *lr, int h, double *nh, int *nTips, double *dm){
+    int i, j, ind;
+    for(i=0; i<*ll; i++){
+        for(j=0; j<*lr; j++){
+            ind = give_index3(left[i], right[j], *nTips);
+            dm[ind] = 2.0*nh[h] - nh[left[i]-1L] - nh[right[j]-1L]; 
+        }   
+    }
+}     
+
+
+void C_coph(int *tips, int *kids, int *ptips, int *pkids, int *ltips, int *lkids, int*Nnode, double *nh, int *nTips, double *dm){
+    int h, j, k, lk, pk, lt, rt, leftk, rightk;
+    for(h=0; h<*Nnode; h++){
+        lk=lkids[h]; 
+        pk=pkids[h];
+        for(j=0; j<(lk-1L); j++){
+            leftk=kids[pk+j] - 1L;
+            lt=ptips[leftk];
+            for(k=j+1L; k<lk; k++) {
+                rightk=kids[pk+k] - 1L;
+                rt = ptips[rightk];
+                copheneticHelp(&tips[lt], &tips[rt], &ltips[leftk], &ltips[rightk], (*nTips+h), nh, nTips, dm);
+            }
+        }
+    }
+}
+
+
+void C_cophenetic(int *children, int *parents, double *el, int *lp, int *m, int *nTips, int *nNode, double *res){
+    double *nh, maxNH; 
+    int i, lt; 
+    int *kids, *lkids, *pkids;
+    int *tips, *ltips, *ptips;
+    nh = (double *) calloc(*m, sizeof(double)); 
+    kids = (int *) R_alloc(*lp, sizeof(int));
+    lkids = (int *) R_alloc(*nNode + 1L, sizeof(int));
+    pkids = (int *) R_alloc(*nNode, sizeof(int));
+    ltips = (int *) R_alloc(*m, sizeof(int));
+    ptips = (int *) R_alloc(*m + 1L, sizeof(int));
+    //nodeH(int *edge, int *node, double *el, int *l,  double *res)
+    nodeH(children, parents, el, lp,  nh);
+    maxNH=nh[0];
+    for(i=1; i<*m; i++)if(maxNH<nh[i]) maxNH=nh[i];
+    for(i=0; i<*m; i++)nh[i] = maxNH - nh[i]; 
+//    tmp <- .C("AllKids", kids, parents, nTips, nNode, lp, integer(lp), integer(nNode+1L),
+//              integer(nNode))
+// void AllKids(int *children, int *parents, int *nTips, int *nNode, int *lp, int *kids, int *lkids, int *pkids){
+    AllKids(children, parents, nTips, nNode, lp, kids, lkids, pkids);
+//tmp2 = .C("C_bipHelp", parents, kids, nTips, m, lp, integer(m), integer(m+1L))
+    C_bipHelp(parents, children, nTips, m, lp, ltips, ptips);
+// tips <- .C("C_bip2", parents, kids, nTips, m, lp, ltips=tmp2[[6]], ptips=tmp2[[7]], integer(sum(tmp2[[6]])))[[8]]    
+    lt = 0;
+    for(i=0; i<*m; i++)lt += ltips[i];
+    tips = (int *) R_alloc(lt, sizeof(int));
+    C_bip2(parents, children, nTips, m, lp, ltips, ptips, tips);
+    //void coph(int *tips, int *kids, int *ptips, int *pkids, int *ltips, int *lkids, int*Nnode, double *nh, int *nTips, double *dm)
+    C_coph(tips, kids, ptips, lkids, ltips, pkids, nNode, nh, nTips, res);
+}
+
+
+// a bit faster 
+SEXP C_bip(SEXP parent, SEXP child, SEXP nTips, SEXP maxP){ //, SEXP Nnode){
+   int eins=1L, i, j, k, l=length(child), *tmp, *tmp2, *lch, *kl, pi, ci, p, nt=INTEGER(nTips)[0], mp=INTEGER(maxP)[0], ltmp; 
+   SEXP ans, ktmp;
+   tmp = (int *) R_alloc(mp, sizeof(int));
+   tmp2 = (int *) R_alloc(mp, sizeof(int));
+   lch = (int *) R_alloc(mp+1L, sizeof(int));
+   kl = (int *) R_alloc(mp+1L, sizeof(int));
+   PROTECT(ans = allocVector(VECSXP, mp)); //INTEGER(Nnode)[0])); 
+   for(i=0; i<nt; i++) SET_VECTOR_ELT(ans, i, ScalarInteger(i+1L)); 
+   p=INTEGER(parent)[0];
+   pi = INTEGER(parent)[1];
+   k=0L;
+   kl[p]=0;
+   lch[p]=1;
+   tmp[0] = INTEGER(child)[0]; 
+   ltmp=1L;
+   for(i=1; i<l; i++){ 
+        pi = INTEGER(parent)[i]; 
+        ci = INTEGER(child)[i];
+        if(pi==p){
+             if(ci < (nt+1L)){
+                 cisort(&ci, tmp, &eins, &ltmp, tmp2);            
+                 ltmp += 1L;
+                 for(j=0; j<ltmp; j++) tmp[j] = tmp2[j];
+             }
+             else{
+                 cisort(INTEGER(VECTOR_ELT(ans, ci-1L)), tmp, &(lch[ci]), &ltmp, tmp2);                       
+                 ltmp += lch[ci]; 
+                 for(j=0; j<ltmp; j++) tmp[j] = tmp2[j];                                
+             }
+             kl[pi]=k; 
+             lch[pi] = ltmp;
+        }  
+        else{
+            PROTECT(ktmp = allocVector(INTSXP, ltmp));
+            for(j=0; j<ltmp; j++)INTEGER(ktmp)[j] = tmp2[j];
+// k???           
+            SET_VECTOR_ELT(ans, p-1L, ktmp); 
+            UNPROTECT(1); // ktmp
+
+            if(ci < (nt+1)){ 
+                 tmp[0]=ci;
+                 ltmp=1L; 
+            } 
+            else{ 
+                ltmp=lch[ci];
+                for(j=0; j<ltmp; j++)tmp[j] = INTEGER(VECTOR_ELT(ans, ci-1L))[j];
+            }
+            k += 1L; 
+            p = pi;
+        }
+   }
+   PROTECT(ktmp = allocVector(INTSXP, ltmp));// mp+1L
+   for(j=0; j<ltmp; j++)INTEGER(ktmp)[j] = tmp2[j];
+   SET_VECTOR_ELT(ans, pi-1L, ktmp);
+   UNPROTECT(2);
+   return(ans);  
+}
 
 
 SEXP C_bipart(SEXP parent, SEXP child, SEXP nTips, SEXP maxP){ //, SEXP Nnode){
@@ -607,6 +899,7 @@ SEXP C_bipart(SEXP parent, SEXP child, SEXP nTips, SEXP maxP){ //, SEXP Nnode){
 // Nnode  
    PROTECT(ans = allocVector(VECSXP, nnode)); //INTEGER(Nnode)[0]));  
    p=INTEGER(parent)[0];
+   pi=INTEGER(parent)[1];
    k=0L;
    kl[p]=0;
    lch[p]=1;
